@@ -2,7 +2,7 @@
 /*
     bqaudioio
 
-    Copyright 2007-2016 Particular Programs Ltd.
+    Copyright 2007-2021 Particular Programs Ltd.
 
     Permission is hereby granted, free of charge, to any person
     obtaining a copy of this software and associated documentation
@@ -122,7 +122,7 @@ static bool initialise() {
             paio_working = true;
         }
     }
-
+    
     return paio_working;
 }
 
@@ -162,11 +162,11 @@ getDeviceNames(bool record)
         const PaDeviceInfo *info = Pa_GetDeviceInfo(i);
 
         ostringstream os;
-        os << "device " << i << " of " << count << ":" << endl;
-        os << "name = \"" << info->name << "\"" << endl;
-        os << "maxInputChannels = " << info->maxInputChannels << endl;
-        os << "maxOutputChannels = " << info->maxOutputChannels << endl;
-        os << "defaultSampleRate = " << info->defaultSampleRate;
+        os << i+1 << "/" << count << ": "
+           << "\"" << info->name << "\","
+           << " channels in " << info->maxInputChannels
+           << ", out " << info->maxOutputChannels
+           << ", default rate " << info->defaultSampleRate;
         log(os.str());
 
         if (record) {
@@ -402,7 +402,7 @@ PortAudioIO::PortAudioIO(Mode mode,
         os << "block size " << m_bufferSize;
         log(os.str());
     }
-
+    
     if (m_source) {
 	m_source->setSystemPlaybackBlockSize(m_bufferSize);
 	m_source->setSystemPlaybackSampleRate(int(round(m_sampleRate)));
@@ -411,6 +411,9 @@ PortAudioIO::PortAudioIO(Mode mode,
     }
 
     if (m_target) {
+        if (m_target->getApplicationChannelCount() == 0) {
+            m_targetChannels = m_inputChannels;
+        }
 	m_target->setSystemRecordBlockSize(m_bufferSize);
 	m_target->setSystemRecordSampleRate(int(round(m_sampleRate)));
 	m_target->setSystemRecordLatency(m_inputLatency);
@@ -433,7 +436,17 @@ PortAudioIO::PortAudioIO(Mode mode,
 	return;
     }
 
-    log("started successfully");
+    ostringstream os;
+    os << "started successfully: rate " << m_sampleRate
+       << ", device input channels " << m_inputChannels
+       << " to record target channels " << m_targetChannels
+       << ", play source channels " << m_sourceChannels
+       << " to device output channels " << m_outputChannels
+       << ", buffer channels " << m_bufferChannels
+       << ", buffer size " << m_bufferSize
+       << ", input latency " << m_inputLatency
+       << ", output latency " << m_outputLatency;
+    log(os.str());
 }
 
 PortAudioIO::~PortAudioIO()
@@ -560,8 +573,12 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
                      const PaStreamCallbackTimeInfo *,
                      PaStreamCallbackFlags)
 {
-#ifdef DEBUG_AUDIO_PORT_AUDIO_IO    
-    cout << "PortAudioIO::process(" << pa_nframes << ")" << endl;
+#ifdef DEBUG_AUDIO_PORT_AUDIO_IO
+    {
+        ostringstream os;
+        os << "process: " << pa_nframes << " frames";
+        log(os.str());
+    }
 #endif
 
     if (!m_prioritySet) {
@@ -577,6 +594,15 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
     int nframes = int(pa_nframes);
 
     if (nframes > m_bufferSize) {
+#ifdef DEBUG_AUDIO_PORT_AUDIO_IO
+        {
+            ostringstream os;
+            os << "extending channel buffers from "
+               << m_bufferSize << " to " << nframes << " frames (across "
+               << m_bufferChannels << " channel(s))";
+            log(os.str());
+        }
+#endif
         m_buffers = reallocate_and_zero_extend_channels
             (m_buffers,
              m_bufferChannels, m_bufferSize,
@@ -591,6 +617,10 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
 
     if (m_target && input) {
 
+#ifdef DEBUG_AUDIO_PORT_AUDIO_IO
+        log("have input and a record target, recording");
+#endif
+
         v_deinterleave
             (m_buffers, input, m_inputChannels, nframes);
         
@@ -602,9 +632,8 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
         for (int c = 0; c < m_targetChannels && c < 2; ++c) {
             float peak = 0.f;
             for (int i = 0; i < nframes; ++i) {
-                if (m_buffers[c][i] > peak) {
-                    peak = m_buffers[c][i];
-                }
+                float sample = fabsf(m_buffers[c][i]);
+                if (sample > peak) peak = sample;
             }
             if (c == 0) peakLeft = peak;
             if (c > 0 || m_targetChannels == 1) peakRight = peak;
@@ -616,9 +645,26 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
 
     if (m_source && output) {
 
-        int received = m_source->getSourceSamples(m_buffers, m_sourceChannels, nframes);
+#ifdef DEBUG_AUDIO_PORT_AUDIO_IO
+        log("have output and a playback source, playing");
+#endif
+
+        int received = m_source->getSourceSamples
+            (m_buffers, m_sourceChannels, nframes);
+
+#ifdef DEBUG_AUDIO_PORT_AUDIO_IO
+        {
+            ostringstream os;
+            os << "received " << received << " frames from application source";
+            log(os.str());
+        }
+#endif
 
         if (received < nframes) {
+            ostringstream os;
+            os << "WARNING: requested " << nframes
+               << " from application source, received only " << received;
+            log(os.str());
             for (int c = 0; c < m_sourceChannels; ++c) {
                 v_zero(m_buffers[c] + received, nframes - received);
             }
@@ -627,7 +673,8 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
         v_reconfigure_channels_inplace
             (m_buffers, m_outputChannels, m_sourceChannels, nframes);
 
-        auto gain = Gains::gainsFor(m_outputGain, m_outputBalance, m_outputChannels);
+        auto gain = Gains::gainsFor
+            (m_outputGain, m_outputBalance, m_outputChannels);
         for (int c = 0; c < m_outputChannels; ++c) {
             v_scale(m_buffers[c], gain[c], nframes);
         }
@@ -636,9 +683,8 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
         for (int c = 0; c < m_outputChannels && c < 2; ++c) {
             float peak = 0.f;
             for (int i = 0; i < nframes; ++i) {
-                if (m_buffers[c][i] > peak) {
-                    peak = m_buffers[c][i];
-                }
+                float sample = fabsf(m_buffers[c][i]);
+                if (sample > peak) peak = sample;
             }
             if (c == 0) peakLeft = peak;
             if (c == 1 || m_outputChannels == 1) peakRight = peak;
@@ -646,10 +692,10 @@ PortAudioIO::process(const void *inputBuffer, void *outputBuffer,
 
         v_interleave
             (output, m_buffers, m_outputChannels, nframes);
-        
+
         m_source->setOutputLevels(peakLeft, peakRight);
 
-    } else if (m_outputChannels > 0) {
+    } else if (m_outputChannels > 0 && output) {
 
         v_zero(output, m_outputChannels * nframes);
     }
