@@ -13,26 +13,30 @@
     COPYING included with this distribution for more information.
 */
 
-#ifndef _POWER_OF_SQRT_TWO_ZOOM_CONSTRAINT_H_
-#define _POWER_OF_SQRT_TWO_ZOOM_CONSTRAINT_H_
+#ifndef SV_POWER_OF_SQRT_TWO_ZOOM_CONSTRAINT_H
+#define SV_POWER_OF_SQRT_TWO_ZOOM_CONSTRAINT_H
 
 #include "base/ZoomConstraint.h"
 
+namespace sv {
+
 class PowerOfSqrtTwoZoomConstraint : virtual public ZoomConstraint
 {
 public:
-    virtual int getNearestBlockSize(int requestedBlockSize,
-                                       RoundingDirection dir = RoundNearest)
-        const;
-    
-    virtual int getNearestBlockSize(int requestedBlockSize,
-                                       int &type,
-                                       int &power,
-                                       RoundingDirection dir = RoundNearest)
-        const;
-        
+    virtual ZoomLevel getNearestZoomLevel(ZoomLevel requested,
+                                          RoundingDirection dir = RoundNearest)
+	const override;
+	
     virtual int getMinCachePower() const { return 6; }
+
+    virtual int getNearestBlockSize(int requestedBlockSize,
+                                    int &type,
+                                    int &power,
+                                    RoundingDirection dir = RoundNearest)
+        const;
 };
 
+} // end namespace sv
+
 #endif
 
