@@ -34,6 +34,14 @@
 
 #include <cmath>
 
+namespace sv {
+
+Layer::ScaleExtents
+Layer::NO_VERTICAL_EXTENTS = {
+    Layer::ScaleApplication::None,
+    CoordinateScale(CoordinateScale::Direction::Vertical, "", false, 0.0, 0.0)
+};
+
 Layer::Layer() :
     m_haveDraggingRect(false),
     m_haveCurrentMeasureRect(false)
@@ -46,19 +54,34 @@ Layer::~Layer()
 }
 
 void
-Layer::connectSignals(const Model *model)
+Layer::connectSignals(ModelId modelId)
 {
-    connect(model, SIGNAL(modelChanged()),
-            this, SIGNAL(modelChanged()));
+    auto model = ModelById::get(modelId);
+    if (!model) return;
+    
+    connect(model.get(), SIGNAL(modelChanged(ModelId)),
+            this, SIGNAL(modelChanged(ModelId)));
 
-    connect(model, SIGNAL(modelChangedWithin(sv_frame_t, sv_frame_t)),
-            this, SIGNAL(modelChangedWithin(sv_frame_t, sv_frame_t)));
+    connect(model.get(), SIGNAL(modelChangedWithin(ModelId, sv_frame_t, sv_frame_t)),
+            this, SIGNAL(modelChangedWithin(ModelId, sv_frame_t, sv_frame_t)));
 
-    connect(model, SIGNAL(completionChanged()),
-            this, SIGNAL(modelCompletionChanged()));
+    connect(model.get(), SIGNAL(completionChanged(ModelId)),
+            this, SIGNAL(modelCompletionChanged(ModelId)));
 
-    connect(model, SIGNAL(alignmentCompletionChanged()),
-            this, SIGNAL(modelAlignmentCompletionChanged()));
+    connect(model.get(), SIGNAL(alignmentCompletionChanged(ModelId)),
+            this, SIGNAL(modelAlignmentCompletionChanged(ModelId)));
+}
+
+ModelId
+Layer::getSourceModel() const
+{
+    ModelId sourceId;
+    auto model = ModelById::get(getModel());
+    while (model && !model->getSourceModel().isNone()) {
+        sourceId = model->getSourceModel();
+        model = ModelById::get(sourceId);
+    }
+    return sourceId;
 }
 
 QString
@@ -74,6 +97,12 @@ Layer::setPresentationName(QString name)
     m_presentationName = name;
 }
 
+bool
+Layer::isPresentationNameSet() const
+{
+    return (m_presentationName != "");
+}
+
 QString
 Layer::getLayerPresentationName() const
 {
@@ -84,7 +113,8 @@ Layer::getLayerPresentationName() const
         (factory->getLayerType(this));
 
     QString modelName;
-    if (getModel()) modelName = getModel()->objectName();
+    auto model = ModelById::get(getModel());
+    if (model) modelName = model->objectName();
             
     QString text;
     if (modelName != "") {
@@ -103,15 +133,11 @@ Layer::setObjectName(const QString &name)
     emit layerNameChanged();
 }
 
-PlayParameters *
+std::shared_ptr<PlayParameters>
 Layer::getPlayParameters() 
 {
-//    cerr << "Layer (" << this << ", " << objectName() << ")::getPlayParameters: model is "<< getModel() << endl;
-    const Model *model = getModel();
-    if (model) {
-        return PlayParameterRepository::getInstance()->getPlayParameters(model);
-    }
-    return nullptr;
+    return PlayParameterRepository::getInstance()->getPlayParameters
+        (getModel().untyped);
 }
 
 void
@@ -143,10 +169,10 @@ Layer::getXScaleValue(const LayerGeometryProvider *v, int x, double &value, QStr
 {
     if (!hasTimeXAxis()) return false;
 
-    const Model *m = getModel();
-    if (!m) return false;
+    auto model = ModelById::get(getModel());
+    if (!model) return false;
 
-    value = double(v->getFrameForX(x)) / m->getSampleRate();
+    value = double(v->getFrameForX(x)) / model->getSampleRate();
     unit = "s";
     return true;
 }
@@ -168,10 +194,9 @@ Layer::getYScaleDifference(const LayerGeometryProvider *v, int y0, int y1,
 sv_frame_t
 Layer::alignToReference(LayerGeometryProvider *v, sv_frame_t frame) const
 {
-    const Model *m = getModel();
-    SVDEBUG << "Layer::alignToReference(" << frame << "): model = " << m << ", alignment reference = " << (m ? m->getAlignmentReference() : nullptr) << endl;
-    if (m && m->getAlignmentReference()) {
-        return m->alignToReference(frame);
+    auto model = ModelById::get(getModel());
+    if (model && !model->getAlignmentReference().isNone()) {
+        return model->alignToReference(frame);
     } else {
         return v->getView()->alignToReference(frame);
     }
@@ -180,10 +205,9 @@ Layer::alignToReference(LayerGeometryProvider *v, sv_frame_t frame) const
 sv_frame_t
 Layer::alignFromReference(LayerGeometryProvider *v, sv_frame_t frame) const
 {
-    const Model *m = getModel();
-    SVDEBUG << "Layer::alignFromReference(" << frame << "): model = " << m << ", alignment reference = " << (m ? m->getAlignmentReference() : nullptr) << endl;
-    if (m && m->getAlignmentReference()) {
-        return m->alignFromReference(frame);
+    auto model = ModelById::get(getModel());
+    if (model && !model->getAlignmentReference().isNone()) {
+        return model->alignFromReference(frame);
     } else {
         return v->getView()->alignFromReference(frame);
     }
@@ -232,7 +256,7 @@ Layer::clipboardHasDifferentAlignment(LayerGeometryProvider *v, const Clipboard
     // We either want to be literal all the way through, or aligned
     // all the way through.
 
-    for (Clipboard::PointList::const_iterator i = clip.getPoints().begin();
+    for (EventVector::const_iterator i = clip.getPoints().begin();
          i != clip.getPoints().end(); ++i) {
 
         // In principle, we want to know whether the aligned version
@@ -252,12 +276,12 @@ Layer::clipboardHasDifferentAlignment(LayerGeometryProvider *v, const Clipboard
         
         sv_frame_t sourceFrame = i->getFrame();
         sv_frame_t referenceFrame = sourceFrame;
-        if (i->haveReferenceFrame()) {
+        if (i->hasReferenceFrame()) {
             referenceFrame = i->getReferenceFrame();
         }
         sv_frame_t myMappedFrame = alignToReference(v, sourceFrame);
 
-//        cerr << "sourceFrame = " << sourceFrame << ", referenceFrame = " << referenceFrame << " (have = " << i->haveReferenceFrame() << "), myMappedFrame = " << myMappedFrame << endl;
+//        cerr << "sourceFrame = " << sourceFrame << ", referenceFrame = " << referenceFrame << " (have = " << i->hasReferenceFrame() << "), myMappedFrame = " << myMappedFrame << endl;
 
         if (myMappedFrame != referenceFrame) return true;
     }
@@ -314,7 +338,7 @@ Layer::MeasureRect::toXml(QTextStream &stream, QString indent) const
 }
 
 void
-Layer::addMeasurementRect(const QXmlAttributes &attributes)
+Layer::addMeasurementRect(const LayerAttributes &attributes)
 {
     MeasureRect rect;
     QString fs = attributes.value("startFrame");
@@ -381,15 +405,15 @@ Layer::measureStart(LayerGeometryProvider *v, QMouseEvent *e)
     // once on measureStart, and then not modify it on drag (to avoid
     // drift from repeated conversion back and forth).
     
-    m_draggingRect.pixrect = QRect(e->x(), e->y(), 0, 0);
+    m_draggingRect.pixrect = QRect(e->position().x(), e->position().y(), 0, 0);
 
     if (m_draggingRect.haveFrames) {
-        m_draggingRect.startFrame = v->getFrameForX(e->x());
-        m_draggingRect.endFrame = v->getFrameForX(e->x());
+        m_draggingRect.startFrame = v->getFrameForX(e->position().x());
+        m_draggingRect.endFrame = v->getFrameForX(e->position().x());
     }
 
-    setMeasureRectYCoord(v, m_draggingRect, true, e->y());
-    setMeasureRectYCoord(v, m_draggingRect, false, e->y());
+    setMeasureRectYCoord(v, m_draggingRect, true, e->position().y());
+    setMeasureRectYCoord(v, m_draggingRect, false, e->position().y());
 
     m_haveDraggingRect = true;
 }
@@ -399,15 +423,15 @@ Layer::measureDrag(LayerGeometryProvider *v, QMouseEvent *e)
 {
     if (!m_haveDraggingRect) return;
 
-    m_draggingRect.pixrect.setHeight(e->y() - m_draggingRect.pixrect.y());
+    m_draggingRect.pixrect.setHeight(e->position().y() - m_draggingRect.pixrect.y());
 
     if (m_draggingRect.haveFrames) {
-        m_draggingRect.endFrame = v->getFrameForX(e->x());
+        m_draggingRect.endFrame = v->getFrameForX(e->position().x());
     } else {
-        m_draggingRect.pixrect.setWidth(e->x() - m_draggingRect.pixrect.x());
+        m_draggingRect.pixrect.setWidth(e->position().x() - m_draggingRect.pixrect.x());
     }
 
-    setMeasureRectYCoord(v, m_draggingRect, false, e->y());
+    setMeasureRectYCoord(v, m_draggingRect, false, e->position().y());
 }
 
 void
@@ -609,28 +633,12 @@ Layer::paintMeasurementRect(LayerGeometryProvider *v, QPainter &paint,
 }
 
 bool
-Layer::valueExtentsMatchMine(LayerGeometryProvider *v) const
+Layer::verticalExtentsMatchMine(LayerGeometryProvider *v) const
 {
-    double min, min_;
-    double max, max_;
-    bool logarithmic, logarithmic_;
-    QString unit;
+    CoordinateScale mine = getVerticalExtents().second;
+    CoordinateScale theirs = v->getEffectiveVerticalExtents(mine.getUnit());
 
-    if (!getValueExtents(min_, max_, logarithmic_, unit)) {
-        return false;
-    }
-
-    if (!v->getValueExtents(unit, min, max, logarithmic)) {
-        return false;
-    }
-
-    if (min != min_ ||
-        max != max_ ||
-        logarithmic != logarithmic_) {
-        return false;
-    }
-
-    return true;
+    return mine.visualRangeMatches(theirs);
 }
 
 void
@@ -644,12 +652,16 @@ Layer::toXml(QTextStream &stream,
             .arg(extraAttributes).arg(encodeEntities(m_presentationName));
     }
 
+    int modelExportId = -1;
+    auto model = ModelById::get(getModel());
+    if (model) modelExportId = model->getExportId();
+    
     stream << QString("<layer id=\"%2\" type=\"%1\" name=\"%3\" model=\"%4\" %5")
         .arg(encodeEntities(LayerFactory::getInstance()->getLayerTypeName
                             (LayerFactory::getInstance()->getLayerType(this))))
-        .arg(getObjectExportId(this))
+        .arg(getExportId())
         .arg(encodeEntities(objectName()))
-        .arg(getObjectExportId(getModel()))
+        .arg(modelExportId)
         .arg(extraAttributes);
 
     if (m_measureRects.empty()) {
@@ -678,12 +690,18 @@ Layer::toBriefXml(QTextStream &stream,
             .arg(extraAttributes).arg(encodeEntities(m_presentationName));
     }
 
+    int modelExportId = -1;
+    auto model = ModelById::get(getModel());
+    if (model) modelExportId = model->getExportId();
+
     stream << QString("<layer id=\"%2\" type=\"%1\" name=\"%3\" model=\"%4\" %5/>\n")
         .arg(encodeEntities(LayerFactory::getInstance()->getLayerTypeName
                             (LayerFactory::getInstance()->getLayerType(this))))
-        .arg(getObjectExportId(this))
+        .arg(getExportId())
         .arg(encodeEntities(objectName()))
-        .arg(getObjectExportId(getModel()))
+        .arg(modelExportId)
         .arg(extraAttributes);
 }
 
+} // end namespace sv
+
